@@ -1,8 +1,8 @@
 use crate::convert::*;
 use crate::operations::*;
+use crate::random_state::PI;
 use crate::RandomState;
 use core::hash::Hasher;
-use crate::random_state::PI;
 
 /// A `Hasher` for hashing an arbitrary stream of bytes.
 ///
@@ -68,7 +68,6 @@ impl AHasher {
         }
     }
 
-
     #[inline]
     pub(crate) fn from_random_state(rand_state: &RandomState) -> Self {
         let key1 = [rand_state.k0, rand_state.k1].convert();
@@ -128,7 +127,11 @@ impl Hasher for AHasher {
     }
 
     #[inline]
-    #[cfg(any(target_pointer_width = "64", target_pointer_width = "32", target_pointer_width = "16"))]
+    #[cfg(any(
+        target_pointer_width = "64",
+        target_pointer_width = "32",
+        target_pointer_width = "16"
+    ))]
     fn write_usize(&mut self, i: usize) {
         self.write_u64(i as u64);
     }
@@ -161,10 +164,10 @@ impl Hasher for AHasher {
                     let tail = data.read_last_u128x4();
                     let mut current: [u128; 4] = [self.key; 4];
                     current[0] = aesenc(current[0], tail[0]);
-                    current[1] = aesenc(current[1], tail[1]);
+                    current[1] = aesdec(current[1], tail[1]);
                     current[2] = aesenc(current[2], tail[2]);
-                    current[3] = aesenc(current[3], tail[3]);
-                    let mut sum: [u128; 2] = [self.key, self.key];
+                    current[3] = aesdec(current[3], tail[3]);
+                    let mut sum: [u128; 2] = [self.key, !self.key];
                     sum[0] = add_by_64s(sum[0].convert(), tail[0].convert()).convert();
                     sum[1] = add_by_64s(sum[1].convert(), tail[1].convert()).convert();
                     sum[0] = shuffle_and_add(sum[0], tail[2]);
@@ -181,8 +184,9 @@ impl Hasher for AHasher {
                         sum[1] = shuffle_and_add(sum[1], blocks[3]);
                         data = rest;
                     }
-                    self.hash_in_2(aesenc(current[0], current[1]), aesenc(current[2], current[3]));
-                    self.hash_in(add_by_64s(sum[0].convert(), sum[1].convert()).convert());
+                    self.hash_in_2(current[0], current[1]);
+                    self.hash_in_2(current[2], current[3]);
+                    self.hash_in_2(sum[0], sum[1]);
                 } else {
                     //len 33-64
                     let (head, _) = data.read_u128x2();
@@ -206,7 +210,7 @@ impl Hasher for AHasher {
     fn finish(&self) -> u64 {
         let combined = aesdec(self.sum, self.enc);
         let result: [u64; 2] = aesenc(aesenc(combined, self.key), combined).convert();
-        result[0]
+        result[1]
     }
 }
 
@@ -317,7 +321,7 @@ pub(crate) struct AHasherStr(pub AHasher);
 impl Hasher for AHasherStr {
     #[inline]
     fn finish(&self) -> u64 {
-        let result : [u64; 2] = self.0.enc.convert();
+        let result: [u64; 2] = self.0.enc.convert();
         result[0]
     }
 
@@ -428,4 +432,3 @@ mod tests {
         assert_eq!(bytes, 0x6464646464646464);
     }
 }
-
